import json
import jsons
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import logging
import requests
import os
import yaml


root_dir = os.path.abspath(os.path.dirname(__file__))
yaml_path = os.path.join(root_dir, '../askdata/askdata_config/base_url.yaml')
with open(yaml_path, 'r') as file:
    url_list = yaml.load(file, Loader=yaml.FullLoader)


def query_to_sql(smartquery, db_driver):
    # Google Pod
    headers = {
        "Content-Type": "application/json"
    }

    s = requests.Session()
    s.keep_alive = False
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
    s.mount('https://', HTTPAdapter(max_retries=retries))

    url = url_list['BASE_URL_QUERY2SQL_DEV'] + "/query_to_sql"

    stringed_smartquery = jsons.dumps(smartquery, strip_nulls=True)
    smartquery = json.loads(stringed_smartquery)

    data = {
        "smartquery": smartquery,
        "db_driver": db_driver
    }

    r = s.post(url=url, headers=headers, json=data)
    r.raise_for_status()

    try:
        dict_response = r.json()
        translation = dict_response['translation']
        return translation
    except Exception as e:
        logging.error(str(e))


def query_to_olap(nl, schema=None, df=None, db_driver="olap"):

    if not (schema is None and df is None):

        # Google Pod
        headers = {
            "Content-Type": "application/json"
        }

        if df is not None:
            dict_df = df.to_dict()
        else:
            dict_df = {}

        if schema is None:
            schema = {}

        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_QUERY2SQL_DEV'] + "/query_to_olap"

        data = {
            "nl_query": nl,
            "table_schema": schema,
            "df": dict_df
        }

        r = s.post(url=url, headers=headers, json=data)
        r.raise_for_status()

        try:
            dict_response = r.json()
            nl_ner = dict_response['nl_ner']
            return nl_ner
        except Exception as e:
            logging.error(str(e))
    else:
        print("Schema is empty and DataFrame is empty.")
        return None


def prepare_ner(nl, schema=None, df=None, db_driver="olap"):

    if not (schema is None and df is None):

        # Google Pod
        headers = {
            "Content-Type": "application/json"
        }

        if df is not None:
            dict_df = df.to_dict()
        else:
            dict_df = {}

        if schema is None:
            schema = {}

        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_QUERY2SQL_DEV'] + "/prepare_ner"

        data = {
            "nl_query": nl,
            "table_schema": schema,
            "df": dict_df
        }

        r = s.post(url=url, headers=headers, json=data)
        r.raise_for_status()

        try:
            dict_response = r.json()
            nl_ner = dict_response['nl_ner']
            return nl_ner
        except Exception as e:
            logging.error(str(e))
    else:
        print("Schema is empty and DataFrame is empty.")
        return None


def query_to_smartquery(query):

    if query is not None:

        # Google Pod
        headers = {
            "Content-Type": "application/json"
        }

        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_QUERY2SQL_DEV'] + "/query_to_smartquery"

        data = {
            "query": query
        }

        r = s.post(url=url, headers=headers, json=data)
        r.raise_for_status()

        try:
            dict_response = r.json()
            smartquery = dict_response['smartquery']
            return smartquery
        except Exception as e:
            logging.error(str(e))
    else:
        print("Schema is empty and DataFrame is empty.")
        return None


def smartquery_to_olap(smartquery):

    if smartquery is not None:

        # Google Pod
        headers = {
            "Content-Type": "application/json"
        }

        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_QUERY2SQL_DEV'] + "/smartquery_to_olap"

        data = {
            "smartquery": smartquery
        }

        r = s.post(url=url, headers=headers, json=data)
        r.raise_for_status()

        try:
            dict_response = r.json()
            nl_ner = dict_response['nl_ner']
            return nl_ner
        except Exception as e:
            logging.error(str(e))
    else:
        print("Schema is empty and DataFrame is empty.")
        return None



import logging
import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import os
import yaml


root_dir = os.path.abspath(os.path.dirname(__file__))
yaml_path = os.path.join(root_dir, '../askdata/askdata_config/base_url.yaml')
with open(yaml_path, 'r') as file:
    url_list = yaml.load(file, Loader=yaml.FullLoader)


def complex_field_calculator(instruction, dialect):

    if instruction is not "" and dialect is not "":

        headers = {
            "Content-Type": "application/json"
        }

        data = {
            "instruction": instruction,
            "dialect": dialect
        }

        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_CFC_DEV'] + "/complexfieldcalculator"

        r = s.post(url=url, headers=headers, json=data)
        r.raise_for_status()

        try:
            response = r.json()['calculated_field']
            return response
        except Exception as e:
            logging.error(str(e))
            print(e)
            return None
    else:
        print("An input is empty!")
        return None


def complex_filter_calculator(instruction, dialect):

    if instruction is not "" and dialect is not "":

        headers = {
            "Content-Type": "application/json"
        }

        data = {
            "instruction": instruction,
            "dialect": dialect
        }

        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_CFC_DEV'] + "/complexfiltercalculator"

        r = s.post(url=url, headers=headers, json=data)
        r.raise_for_status()

        try:
            response = r.json()['calculated_filter']
            return response
        except Exception as e:
            logging.error(str(e))
            print(e)
            return None
    else:
        print("An input is empty!")
        return None


def smartjoin(tables):

    if bool(tables):

        headers = {
            "Content-Type": "application/json"
        }

        data = {
            "tables": tables
        }

        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_CFC_DEV'] + "/smartjoin"

        r = s.post(url=url, headers=headers, json=data)
        r.raise_for_status()

        try:
            response = r.json()['join_list']
            return response
        except Exception as e:
            logging.error(str(e))
            print(e)
            return None
    else:
        print("The input is empty!")
        return None



from askdata import nlg
import pandas as pd

if __name__ == "__main__":
    df = pd.DataFrame()
    df = df.append({'player': 'Cristiano Ronaldo', 'goals': 10, 'team': 'Juventus'}, ignore_index=True)
    df = df.append({'player': 'Roberto Baggio', 'goals': 2, 'team': 'Milan'}, ignore_index=True)
    examples = [["player=Cristiano Ronaldo, goals=3, team=Juventus", "Cristiano Ronaldo scored 3 goals for Juventus"]]
    res = nlg.text_from_data(df=df, examples=examples, fit_items=False, max_tokens=64)
    print(res)



from askdata import complex_query_calculator

if __name__ == "__main__":
    dialect = "MySQL"
    instruction = "difference between PURCHASE_DATE (date) and INVOICE_DATE (date)"

    # Complex field
    res = complex_query_calculator.complex_field_calculator(instruction=instruction, dialect=dialect)
    print(res)

    # Complex filter
    res = complex_query_calculator.complex_filter_calculator(instruction=instruction, dialect=dialect)
    print(res)

    # Smartjoin
    tables = {
        "table_left":
            {
                "columns": [
                    {"label": "person ID", "type": "numeric"},
                    {"label": "name", "type": "string"},
                    {"label": "surname", "type": "string"},
                    {"label": "work name", "type": "string"},
                    {"label": "salary", "type": "numeric"},
                    {"label": "city ID", "type": "string"}
                ],
                "values": [
                    ["10", "Francesco", "De Gaio", "engineer", "3000", "NA"],
                    ["22", "Giacomo", "Martin", "employer", "1300", "BO"],
                    ["43", "Luigia", "Mazzi", "chef", "1400", "CA"],
                    ["90", "Martina", "Di Giuseppe", "doctor", "2500", "RM"],
                    ["3", "Rindina", "Dindi", "butcher", "1200", "CT"]
                ]
            },
        "table_right":
            {
                "columns":
                    [
                        {"label": "city", "type": "string"},
                        {"label": "state id", "type": "numeric"},
                        {"label": "population number", "type": "numeric"}
                    ],
                "values":
                    [
                        ["RM", "01", "10101"],
                        ["RI", "04", "93727"],
                        ["PL", "12", "2917283"],
                        ["NA", "01", "9172648"],
                        ["ML", "01", "2818"]
                    ]
            }
    }
    res = complex_query_calculator.smartjoin(tables=tables)
    print(res)



import operator
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import random
import logging
import requests
import pandas as pd
import os
import yaml

ops = {
    '<': operator.lt,
    '<=': operator.le,
    '==': operator.eq,
    '!=': operator.ne,
    '>=': operator.ge,
    '>': operator.gt
}


root_dir = os.path.abspath(os.path.dirname(__file__))
yaml_path = os.path.join(root_dir, '../askdata/askdata_config/base_url.yaml')
with open(yaml_path, 'r') as file:
    url_list = yaml.load(file, Loader=yaml.FullLoader)


def get_conditional_phrases(conditions, phrase1, phrase2):
    bool_array = []
    for condition in conditions:
        for op in ops:
            if op in condition:
                splitted = condition.split(' ' + op + ' ')
                operator = ops[op]
                bool_array.append(operator(splitted[0], splitted[1]))
        # bool_array.append(eval(condition))
    if False in bool_array:
        return phrase2
    else:
        return phrase1


def get_random_synonymous(synonyms):
    random.seed()
    nKeys = len(synonyms.keys())
    pickedItem = random.randint(0, nKeys - 1)
    synPicked = synonyms[pickedItem]
    return synPicked


def words_to_digits(phrase):
    words = phrase.split(" ")
    num = []
    for word in words:
        if "." in word:
            word = word.replace(".", "")
        if "," in word:
            word = word.replace(",", "")
        if word.isdigit():
            num.append(int(word))
    return num


def add_random_synonymous_to_sentence(phrase, placeholder, synonyms):
    random.seed()
    nKeys = len(synonyms.keys())
    pickedItem = random.randint(0, nKeys - 1)
    synPicked = synonyms[pickedItem]
    phrase = phrase.replace(placeholder, synPicked)
    return phrase


def text_from_data(df, examples=None, max_tokens=64, fit_items=True):
    # The parameter "df" is a dataframe, but it will be converted into a dict, and called df in each case

    if examples is None:
        examples = [["player=Cristiano Ronaldo, goals=10, team=Juventus", "Cristiano Ronaldo scored 10 goals for Juventus"]]
        print("Using a default example: \n" + str(examples) + "\n")

    if not df.empty:
        headers = {
            "Content-Type": "application/json"
        }

        dict_df = df.to_dict()

        data = {
            "df": dict_df,
            "examples": examples,
            "fit_items": fit_items,
            "max_tokens": max_tokens
        }

        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_CFC_DEV'] + "/nlg"

        r = s.post(url=url, headers=headers, json=data)
        r.raise_for_status()

        try:
            df_response = pd.DataFrame.from_dict(r.json()['dict_df'])
            return df_response
        except Exception as e:
            logging.error(str(e))
    else:
        print("Input DataFrame is empty!")
        df_response = pd.DataFrame()
        return df_response



import logging
import os
import requests
import yaml
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

root_dir = os.path.abspath(os.path.dirname(__file__))
yaml_path = os.path.join(root_dir, '../askdata/askdata_config/base_url.yaml')
with open(yaml_path, 'r') as file:
    url_list = yaml.load(file, Loader=yaml.FullLoader)


def suggest_synonyms(word, lang=None, initial_synonyms_list=None):

    if lang is None:
        lang = "en"
        print("Using English language as default!")

    if initial_synonyms_list is None:
        initial_synonyms_list = []

    # Google Pod
    headers = {
        "Content-Type": "application/json"
    }

    s = requests.Session()
    s.keep_alive = False
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
    s.mount('https://', HTTPAdapter(max_retries=retries))

    url = url_list['BASE_URL_SMART_SYNONYM_PROD']

    data = {
        "word": word,
        "lang": lang,
        "initial_synonyms_list ": initial_synonyms_list
    }

    r = s.post(url=url, headers=headers, json=data)
    r.raise_for_status()

    try:
        dict_response = r.json()
        synonyms = dict_response['synonyms']
        business_name = dict_response['business_name']
        return synonyms, business_name
    except Exception as e:
        logging.error(str(e))



from askdata import query2sql
from askdata.smartquery import SmartQuery, Query, Field, Condition, Sorting, SQLSorting, SQLOperator, TimeOperator, \
    BooleanOperator, CompositeCondition
import pandas as pd

if __name__ == "__main__":

    # Query2SQL
    smartquery = SmartQuery(
        queries=[
            Query(
                fields=[
                    Field(aggregation="SUM", column="incidents", alias="sum_incidents",
                          internalDataType="NUMERIC",
                          sourceDataType="INT64"),
                    Field(column="customer_name", alias="Customer",
                          internalDataType="STRING",
                          sourceDataType="VARCHAR"),
                    Field(aggregation="YEAR", column="acquired", alias="Acquired Date",
                          internalDataType="DATE",
                          sourceDataType="DATE")
                ],
                where=[
                    CompositeCondition(conditions=[
                        Condition(field=Field(column="customer_name", alias="Customer",
                                              internalDataType="STRING",
                                              sourceDataType="VARCHAR"), operator=SQLOperator.IN,
                                  value=["Franceaco Doe", "Claude Rudolf"]),
                        Condition(field=Field(aggregation="YEAR", column="acquired", alias="Acquired Date",
                                              internalDataType="DATE",
                                              sourceDataType="DATE"), operator=TimeOperator.RANGE,
                                  value=["2018-01-01", "2019-12-31"])
                    ], operator=BooleanOperator.AND)

                ],
                orderBy=[
                    Sorting(field="Acquired Date", order=SQLSorting.DESC)
                ],
                limit=6
            )
        ]
    )

    response = query2sql.query_to_sql(smartquery=smartquery, db_driver="MySQL")
    print(response)

    # Query 2 OLAP
    nl = "list all the revenue by player"
    df = ""
    schema = {
        "SerieA.Revenue": {
            "name":"revenue",
            "synonyms": ["revenues"]
        },
        "SerieA.Player": {
            "name": "player",
            "synonyms": ["players", "soccer player"],
            "example_of_values": ["Cristiano Ronaldo", "Ciro Immobile", "Alessandro del Piero"]
        }
    }
    db_driver = "olap"

    # Function call
    result_for_ner = query2sql.prepare_ner(nl=nl, schema=schema, db_driver=db_driver)
    smartquery = query2sql.query_to_smartquery(result_for_ner)
    olap_query = query2sql.smartquery_to_olap(smartquery)

    # Print the DataFrame result
    print("Answer: ")
    print(result_for_ner)
    print("#----#")
    print(smartquery)
    print("#----#")
    print(olap_query)

    # All
    response = query2sql.query_to_olap(nl=nl, schema=schema, db_driver=db_driver)
    print("Answer: ")
    print(response)
